@@ -41,7 +41,7 @@ use Glib::Object::Subclass Gscan2pdf::Dialog::Scan::, properties => [
     ),
 ];
 
-our $VERSION = '2.3.0';
+our $VERSION = '2.5.5';
 
 my $SANE_NAME_SCAN_TL_X   = SANE_NAME_SCAN_TL_X;
 my $SANE_NAME_SCAN_TL_Y   = SANE_NAME_SCAN_TL_Y;
@@ -131,6 +131,7 @@ sub scan_options {
     # Remove lookups to geometry boxes and option widgets
     delete $self->{geometry_boxes};
     delete $self->{option_widgets};
+    delete $self->{option_info};
 
     # Ghost the scan button whilst options being updated
     $self->set_response_sensitive( 'ok', FALSE );
@@ -351,7 +352,8 @@ sub _initialise_options {    ## no critic (ProhibitExcessComplexity)
         else {                                       # $opt->{max_values} > 1
             $widget = Gtk3::Button->new( $d_sane->get( $opt->{title} ) );
             $widget->{signal} = $widget->signal_connect(
-                clicked => \&multiple_values_button_callback,
+                clicked =>
+                  \&Gscan2pdf::Dialog::Scan::multiple_values_button_callback,
                 [ $self, $opt ]
             );
         }
@@ -368,6 +370,30 @@ sub _initialise_options {    ## no critic (ProhibitExcessComplexity)
     return;
 }
 
+sub _post_set_option_hook {
+    my ( $self, $option, $val, $uuid ) = @_;
+
+    # We can carry on applying defaults now, if necessary.
+    $self->signal_emit( 'finished-process',
+        "set_option $option->{name}"
+          . ( $option->{type} == SANE_TYPE_BUTTON ? $EMPTY : " to $val" ) );
+
+    # Unset the profile unless we are actively setting it
+    if ( not @{ $self->{setting_profile} } ) {
+        $self->set( 'profile', undef );
+
+        # Emit the changed-current-scan-options signal
+        # unless we are actively setting it
+        if ( not @{ $self->{setting_current_scan_options} } ) {
+            $self->signal_emit( 'changed-current-scan-options',
+                $self->get('current-scan-options'), $EMPTY );
+        }
+    }
+
+    $self->signal_emit( 'changed-scan-option', $option->{name}, $val, $uuid );
+    return;
+}
+
 # Update the sane option in the thread
 # If necessary, reload the options,
 # and walking the options tree, update the widgets
@@ -399,44 +425,47 @@ sub set_option {
             $self->signal_emit( 'changed-progress', undef, undef );
         },
         finished_callback => sub {
-            my ( $data, $status ) = @_;
+            my ( $info, $status ) = @_;
 
             if ( $status != SANE_STATUS_INVAL ) {
                 $self->{current_scan_options}
                   ->add_backend_option( $option->{name}, $val );
             }
 
-            my $opt;
-            if ($data) {
-                $self->update_options(
-                    Gscan2pdf::Scanner::Options->new_from_data($data) );
-                $opt = $options->by_name( $option->{name} );
+            $self->{option_info}{ $option->{name} } = $info;
+
+            if ( defined $info and $info & SANE_INFO_RELOAD_OPTIONS ) {
+                Gscan2pdf::Frontend::Image_Sane->find_scan_options(
+                    sub {    # started callback
+                        $self->signal_emit( 'started-process',
+                            __('Retrieving options') );
+                    },
+                    sub {    # running callback
+                        $self->signal_emit( 'changed-progress', undef, undef );
+                    },
+                    sub {    # finished callback
+                        my ($data) = @_;
+                        $self->update_options(
+                            Gscan2pdf::Scanner::Options->new_from_data($data) );
+                        $self->_post_set_option_hook( $option, $val, $uuid );
+                    },
+                    sub {    # error callback
+                        my ($message) = @_;
+                        $self->signal_emit(
+                            'process-error',
+                            'find_scan_options',
+                            __(
+                                'Error retrieving scanner options: ' . $message
+                            )
+                        );
+                    }
+                );
             }
             else {
-                $opt = $options->by_name( $option->{name} );
+                my $opt = $options->by_name( $option->{name} );
                 $opt->{val} = $val;
+                $self->_post_set_option_hook( $option, $val, $uuid );
             }
-
-            # We can carry on applying defaults now, if necessary.
-            $self->signal_emit( 'finished-process',
-                "set_option $option->{name}"
-                  . ( $opt->{type} == SANE_TYPE_BUTTON ? $EMPTY : " to $val" )
-            );
-
-            # Unset the profile unless we are actively setting it
-            if ( not @{ $self->{setting_profile} } ) {
-                $self->set( 'profile', undef );
-
-                # Emit the changed-current-scan-options signal
-                # unless we are actively setting it
-                if ( not @{ $self->{setting_current_scan_options} } ) {
-                    $self->signal_emit( 'changed-current-scan-options',
-                        $self->get('current-scan-options'), $EMPTY );
-                }
-            }
-
-            $self->signal_emit( 'changed-scan-option', $option->{name}, $val,
-                $uuid );
         },
         error_callback => sub {
             my ($message) = @_;
